import { useApolloClient } from "@apollo/client";
import * as R from "ramda";
import { FC, lazy, useMemo } from "react";
import {
  Navigate,
  useParams,
  useRoutes,
  useSearchParams,
} from "react-router-dom";
import { ARTICLE } from "./articles";
import { Article } from "./generated/graphql";
import { DefaultLayout } from "./layouts";

const CreateArticle = () => {
  const ArticleEditor = lazy(() =>
    import(
      /* webpackChunkName: "article-editor" */ "./articles/article-editor"
    ).then((m) => ({
      default: m.ArticleEditor,
    }))
  );
  const [search] = useSearchParams();
  if (!search.has("key")) {
    search.set("key", new Date().toISOString());
    return <Navigate to={`/articles/create?${search}`} replace />;
  }
  return <ArticleEditor draftKey={search.get("key")!} />;
};

const ModifyArticle: FC = () => {
  const { id } = useParams();
  const client = useApolloClient();
  const Lazy = useMemo(
    () =>
      lazy(async () => {
        const [{ ArticleEditor }, article] = await Promise.all([
          import(
            /* webpackChunkName: "article-editor" */ "./articles/article-editor"
          ),
          client
            .query<{ article: Article }, { id: string }>({
              query: ARTICLE,
              variables: { id: id! },
            })
            .then(({ data }) => {
              return R.omit(["__typename"], data.article);
            }),
        ]);
        return {
          default: () => (
            <ArticleEditor article={article} draftKey={article.key!} />
          ),
        };
      }),
    [id, client]
  );
  return <Lazy />;
};

const ArticleIndex = lazy(() =>
  import(/* webpackChunkName: "articles" */ "./articles").then((m) => ({
    default: m.ArticleIndex,
  }))
);

const TagIndex = lazy(() =>
  import(/* webpackChunkName: "tags" */ "./tags").then((m) => ({
    default: m.TagsIndex,
  }))
);

export const AppRoutes: FC = () => {
  const element = useRoutes([
    {
      index: true,
      element: <div>DashBoard</div>,
    },
    {
      path: "/articles/create",
      element: (
        <DefaultLayout>
          <CreateArticle />
        </DefaultLayout>
      ),
    },
    {
      path: "/articles/:id",
      element: (
        <DefaultLayout>
          <ModifyArticle />
        </DefaultLayout>
      ),
    },
    {
      path: "articles",
      element: (
        <DefaultLayout>
          <ArticleIndex />
        </DefaultLayout>
      ),
    },
    {
      path: "tags",
      element: (
        <DefaultLayout>
          <TagIndex />
        </DefaultLayout>
      ),
    },
  ]);

  return element;
};
